@@ -1,25 +1,45 @@
-##########
+################################################################################
 # This file contains definitions for the functions to obtain Newey-West based
 # estimates of parameter estimates resulting for the generalized profile
 # proceedure. 
 #
-# The main function is 'Profile.covariance'. Some utility functions are also defined. 
+# The main function is 'Profile.covariance'. 
+#  Utility functions blocks2mat, trimr, Newey.West and NeweyWest.Var are
+#    also defined.
+################################################################################
 
-Profile.covariance <- function(pars,active=NULL,times,data,coefs,lik,proc,in.meth='nlminb',control.in=NULL,eps=1e-6,GN=FALSE)
+Profile.covariance <- function(pars,active=NULL,times,data,coefs,lik,proc,
+                               in.meth='nlminb',control.in=NULL,eps=1e-6,
+                               GN=FALSE)
 {
+    # First, we'll need to allow for repeated measurements
+    if(length(dim(data)) == 3){
+       index = matrix(1:(dim(data)[1]*dim(data)[2]),dim(data)[1],dim(data)[2],
+                      byrow=FALSE)
+       times = rep(x=times,times=dim(data)[3])
+       data = matrix(data,dim(data)[1]*dim(data)[2],dim(data)[3])
+    }
+    else{ index = matrix(1:dim(data)[1],dim(data)[1],1) }
+    if(length(dim(coefs)) == 3){
+       coefs = matrix(coefs,dim(coefs)[1]*dim(coefs)[2],dim(coefs)[3])
+    }
+ 
     check.lik.proc.data.coefs(lik,proc,data,times,coefs)
 
     if(is.null(active)){ active = 1:length(pars) }
 
     apars = pars[active]
    
-    g = ProfileDP(pars=apars,allpars=pars,times=times,data=data,coefs=coefs,lik=lik,proc=proc,active=active,sumlik=FALSE)
+    g = ProfileDP(pars=apars,allpars=pars,times=times,data=data,coefs=coefs,
+                  lik=lik,proc=proc,active=active,sumlik=FALSE)
     if(!is.matrix(g)){ g = matrix(g,length(g),length(active)) }
 
     if(!GN){
 
       H = matrix(0,length(apars),length(apars))
-      gg = ProfileDP(pars=apars,allpars=pars,times=times,data=data,coefs=coefs,lik=lik,proc=proc,active=active,sumlik=TRUE)
+      #  see file OutOptimization for functions ProfileDP and ProfileDP.AllPar
+      gg = ProfileDP(pars=apars,allpars=pars,times=times,data=data,
+                     coefs=coefs,lik=lik,proc=proc,active=active,sumlik=TRUE)
       for(i in 1:length(apars)){
         if(file.exists('curcoefs.tmp')){file.remove('curcoefs.tmp')}
         if(file.exists('optcoefs.tmp')){file.remove('optcoefs.tmp')}
@@ -28,8 +48,10 @@ Profile.covariance <- function(pars,active=NULL,times,data,coefs,lik,proc,in.met
           tpars = apars
           tpars[i] = tpars[i] + eps
 
-          tf = ProfileErr(tpars,pars,times,data,coefs,lik,proc,in.meth=in.meth,control.in=control.in,active=active)
-          tg = ProfileDP(tpars,pars,times,data,coefs,lik,proc,active=active,sumlik=TRUE)
+          tf = ProfileErr(tpars,pars,times,data,coefs,lik,proc,in.meth=in.meth,
+                          control.in=control.in,active=active)
+          tg = ProfileDP(tpars,pars,times,data,coefs,lik,proc,active=active,
+                         sumlik=TRUE)
 
           H[,i] = (tg-gg)/eps
 
@@ -41,19 +63,21 @@ Profile.covariance <- function(pars,active=NULL,times,data,coefs,lik,proc,in.met
     else{
         H = t(g[,active])%*%g[,active]
     }
-           
-    Covar = NeweyWest.Var( 0.5*(t(H)+H) ,g,5)
-            
+    
+    Covar = 0
+    for(i in 1:ncol(index)){       
+      Covar = Covar + NeweyWest.Var( 0.5*(t(H)+H) ,g[index[,i],],5)
+    }        
     return( Covar )
 }
 
 
 
-############################################################################################
+################################################################################
 #
 # Some Utilities
 #
-############################################################################################
+################################################################################
 
 blocks2mat = function(H)   # List of matrices -> large matrix
 {
@@ -88,14 +112,15 @@ blocks2mat = function(H)   # List of matrices -> large matrix
 
     for(i in 1:length(H)){
       for(j in 1:length(H[[i]])){
-        out[(rowdims[i]+1):rowdims[i+1],(coldims[j]+1):coldims[j+1]] = H[[i]][[j]]
+        out[(rowdims[i]+1):rowdims[i+1],(coldims[j]+1):coldims[j+1]] = 
+               H[[i]][[j]]
       }
     }
   }
   return(out)
 }
 
-
+################################################################################
 
 #blocks2mat = function(H)  # List of matrices -> large matrix
 #{
@@ -109,7 +134,8 @@ blocks2mat = function(H)   # List of matrices -> large matrix
 #      if(length(H[[i]])>1){
 #        for(j in 2:length(H[[i]])){
 #        print(c(i,j))
-#            if(inherits(H[[i]][[j]],'dgCMatrix')|inherits(H[[i]][[j]],'dgeMatrix')){ tout=cBind(tout,H[[i]][[j]]) }
+#            if(inherits(H[[i]][[j]],'dgCMatrix')|inherits(H[[i]][[j]],
+#                         'dgeMatrix')){ tout=cBind(tout,H[[i]][[j]]) }
 #            else{ tout = cbind(tout,H[[i]][[j]]) }
 #        }
 #      }
@@ -134,32 +160,37 @@ blocks2mat = function(H)   # List of matrices -> large matrix
 
 ## GAUSS trimr function: trims n1 rows from the start and n2 rows from the end
 ## of a matrix or vector 
+
+################################################################################
+
 trimr <- function (a,n1,n2) {
         da<-dim(a); 
         if(is.null(da)) {a[(n1+1):(length(a)-n2)]}
         else {a[(n1+1):(da[1]-n2),]}
 }
 
+################################################################################
+
 Newey.West <-function(x,y,maxlag) {
         w=1-(1:maxlag)/(maxlag+1); w=w/length(x); 
         out=mean(x*y); 
         for(i in 1:maxlag) {
-            out=out+w[i]*sum(trimr(x,i,0)*trimr(y,0,i))+w[i]*sum(trimr(y,i,0)*trimr(x,0,i))
+            out=out + w[i]*sum(trimr(x,i,0)*trimr(y,0,i)) +
+                      w[i]*sum(trimr(y,i,0)*trimr(x,0,i))
         }
         return(out)     
 } 
 
+################################################################################
+
 NeweyWest.Var = function(H,g,maxlag)       
 {
     V = solve(H)
-
-    I = 0*H
-    
+    I = 0*H    
     if(is.null(maxlag)){ 
         n = nrow(g)
         maxlag = max(5,n^(0.25))
-    }
-              
+    }              
     if(maxlag > 0){
         for(i in 1:ncol(g)){
             for(j in i:ncol(g)){
@@ -169,6 +200,5 @@ NeweyWest.Var = function(H,g,maxlag)
         }
     }
     return( V%*%(I+ t(g)%*%g)%*%V  )
-
 }
 
